package com.ruoyi.app.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ruoyi.app.domain.TbFamilyGarden;
import com.ruoyi.app.service.ITbFamilyGardenService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 家族园地
 *
 * @author liuhj
 * @date 2020-10-14
 */
@Api(value = "家族园地", tags = "家族园地")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/app/garden" )
public class FamilyGardenController extends BaseController {

    private final ITbFamilyGardenService iTbFamilyGardenService;

    /**
     * 查询家族园地列表
     */
    @ApiOperation(value = "家族园地列表", notes = "家族园地列表")
    @GetMapping("/list")
    public TableDataInfo list(TbFamilyGarden tbFamilyGarden)
    {
        startPage();
        List<TbFamilyGarden> list = iTbFamilyGardenService.queryList(tbFamilyGarden);
        return getDataTable(list);
    }


    /**
     * 获取家族园地详细信息
     */
    @ApiOperation(value = "家族园地详细信息", notes = "家族园地详细信息")
    @GetMapping(value = "/{id}" )
    public AjaxResult detail(@PathVariable("id" ) Long id) {
        return AjaxResult.success(iTbFamilyGardenService.getById(id));
    }

    /**
     * 新增家族园地
     */
    @ApiOperation("新增家族园地")
    @Log(title = "家族园地" , businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public AjaxResult add(@RequestBody TbFamilyGarden tbFamilyGarden) {
        Date date = new Date();
        tbFamilyGarden.setCreateTime(date);
        return toAjax(iTbFamilyGardenService.save(tbFamilyGarden) ? 1 : 0);
    }

    /**
     * 修改家族园地
     */
    @ApiOperation("修改家族园地")
    @Log(title = "家族园地" , businessType = BusinessType.UPDATE)
    @PutMapping("/update")
    public AjaxResult edit(@RequestBody TbFamilyGarden tbFamilyGarden) {
        return toAjax(iTbFamilyGardenService.updateById(tbFamilyGarden) ? 1 : 0);
    }

    /**
     * 删除家族园地
     */
    @ApiOperation("删除家族园地")
    @Log(title = "家族园地" , businessType = BusinessType.DELETE)
    @DeleteMapping("/del")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(iTbFamilyGardenService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
