package com.ruoyi.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.app.domain.TbMemberMiddle;
import com.ruoyi.app.domain.vo.AppFamilyVo;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbMemberMiddleService;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.app.mapper.TbFamilyMapper;
import com.ruoyi.app.domain.TbFamily;
import com.ruoyi.app.service.ITbFamilyService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.Date;
import java.util.List;

/**
 * 家族Service业务层处理
 *
 * @author Alex
 * @date 2020-09-28
 */
@Service
public class TbFamilyServiceImpl extends ServiceImpl<TbFamilyMapper, TbFamily> implements ITbFamilyService {

    @Autowired
    private ITbFamilyService familyService;
    @Autowired
    private ITbMemberMiddleService middleService;
    @Autowired
    private ITbAppUserService userService;

    /**
     * 根据会员id获取家族列表
     * @param uid
     * @return
     */
    @Override
    public List<TbFamily> selectByUserId(Long uid) {
        return baseMapper.selectByUserId(uid);
    }
    /**
     * 根据成员id获取家族列表
     * @param mid
     * @return
     */
    @Override
    public List<AppFamilyVo> selectByMemberId(Long mid) {
        return baseMapper.selectByMemberId(mid);
    }

    @Override
    public List<AppFamilyVo> pageList(LambdaQueryWrapper<TbFamily> wrapper){
        return baseMapper.pageList(wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public AjaxResult applyFamily(AppMemberVo memberVo){

        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getFamilyId,memberVo.getFamilyId())
                .eq(TbMemberMiddle::getMemberId,memberVo.getMemberId())
                .last("limit 1")
        );
        if (middle == null) {
            middle = new TbMemberMiddle();
            middle.setFamilyId(memberVo.getFamilyId());
            middle.setMemberId(memberVo.getMemberId());
        } else {
            if ("2".equals(middle.getStatus())){
                return AjaxResult.error("已在该家族，不允许重复加入");
            }
        }

        memberVo.setJoins("Y");
        // 保存成员
        if(!userService.saveOrUpdate(memberVo)){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("申请失败");
        }
        // 是否已加入过家族
        List<TbMemberMiddle> middleList = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getMemberId,memberVo.getMemberId())
        );

        if (middleList.size() == 0) {
            // 如果从未加入过家族，则设置为默认家族
            middle.setAcquiesce("Y");
        }
        middle.setStatus("1"); // 申请
        if(!middleService.saveOrUpdate(middle)){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return AjaxResult.error("申请失败");
        }
        return AjaxResult.success("申请成功");
    }

    @Override
    public boolean saveFamily(TbFamily family){
        if (familyService.save(family)) {
            // TODO 如果是APP端添加，则把添加人设置为家族管理员
            family.setCode(StringUtils.codeFormat(family.getId()));
        }
        return familyService.updateById(family);
    }

    @Override
    public TbFamily myFamily(Long memberId) {
        TbMemberMiddle middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                .eq(TbMemberMiddle::getMemberId, memberId)
                .eq(TbMemberMiddle::getAcquiesce, "Y")
                .last("limit 1")
        );
        if (middle == null) {
            // 没有默认家族，则设置第一个家族为默认
            middle = middleService.getOne(new LambdaQueryWrapper<TbMemberMiddle>()
                    .eq(TbMemberMiddle::getMemberId, memberId)
                    .last("limit 1")
            );
            if (middle == null) {
                return null;
            }
            middle.setAcquiesce("Y");
            middleService.save(middle);
        }
        TbFamily family = familyService.getOne(new LambdaQueryWrapper<TbFamily>()
                .eq(TbFamily::getId, middle.getFamilyId())
                .last("limit 1")
        );
        return family;
    }

    @Override
    public boolean change(Long memberId,Long familyId) {
        List<TbMemberMiddle> middleList = middleService.list(new LambdaQueryWrapper<TbMemberMiddle>()
            .eq(TbMemberMiddle::getMemberId, memberId)
        );
        if (middleList.size() <= 0) {
            return false;
        }
        middleList.forEach(item -> {
            item.setAcquiesce("N");
            if (familyId.equals(item.getFamilyId())) {
                item.setAcquiesce("Y");
            }
        });
        return middleService.updateBatchById(middleList);
    }

}
